<?php
// © Microsoft Corporation. All rights reserved.

namespace microsoft_start\services;

use microsoft_start\infrastructure\Util;

class MSNClient
{
    static function update_post($body)
    {
        return static::BaseRequest("thirdparty/contents/article/{$body['id']}?status=3", ['method' => 'POST', 'body' => $body]);
    }

    static function send_logs($body)
    {
        $token = TokenService::get_token();

        $response = wp_remote_post(
            MSPH_SERVICE_URL . "telemetry?scn=3rdPartyAuth&wrapodata=false",
            [
                'headers' => [
                    "accept" => "*/*",
                    'Authorization' => "Bearer $token",
                    "accept-language" => "en-US,en;q=0.9",
                    "content-type" => "application/json",
                ],
                'method'      => 'POST',
                'body'        => json_encode($body)
            ]
        );
        return $response;
    }

    static function upsert_feed($feedConfig)
    {
        $config = array('auth' => true, 'body' => $feedConfig, 'method' => 'POST', 'timeout' => 10);
        return static::BaseRequest("thirdparty/feeds?handler=upsert", $config);
    }

    static function suspend_feed()
    {
        $config = array('auth' => true, 'body' => (object)array(), 'method' => 'PUT', 'timeout' => 10);
        return static::BaseRequest("thirdparty/feeds?handler=suspend", $config);
    }

    static function account_settings()
    {

        $response = static::BaseRequest("account/settings?applicationVersion=" . MSPH_PLUGIN_VERSION);
        if (!$response || $response['response']['code'] != 200) {
            return null;
        }

        $settings = json_decode($response['body']);
        if ($settings) {
            Options::set_profile(json_encode($settings));
            $lifeCycleStatus = $settings->lifeCycleStatus;
            Options::set_CID($settings->accountId);
            if ($lifeCycleStatus == 0) {
                Options::set_status('active');
            } else if ($lifeCycleStatus == 3) {
                Options::set_status('pending');
            }
        }
        return $settings;
    }

    static function account_profile()
    {

        $response = static::BaseRequest("account/profile");
        if (!$response || $response['response']['code'] != 200) {
            return null;
        }

        $profile = json_decode($response['body']);

        return $profile;
    }

    static function get_health_check_data($body)
    {
        $config = array('auth' => false, 'body' => $body, 'method' => 'POST');
        $response = static::BaseRequest("thirdparty/helpcenter/healthcheck", $config);
        if (!$response || $response['response']['code'] != 200) {
            return null;
        }
        $body = json_decode($response['body'], true);
        return $body;
    }

    static function get_article_publish_status($article_id_list)
    {
        $idListArray = explode(",", $article_id_list);
        $results = array();
        $limitation = 10;       // due to limitation from Http.sys registry settings for Windows.

        while (!empty($idListArray)) {
            $selectedIdListArray = array_slice($idListArray, 0, $limitation);
            $selectedIdList = implode(",", $selectedIdListArray);
            $encodedSelectedIdList = urlencode($selectedIdList);
            $url = "thirdparty/contents/article/?ids={$encodedSelectedIdList}&res=publish";
            $response = static::BaseRequest($url, ['method' => 'GET']);
            if ($response && $response['response']['code'] >= 200 && $response['response']['code'] < 300) {
                array_push($results, json_decode($response['body']));
            }
            array_splice($idListArray, 0, $limitation);
        }

        $totalResult = call_user_func_array('array_merge', $results);
        return $totalResult == null ? [] : $totalResult;
    }
    static function get_wp_notification()
    {
        // 1. Microsoft Start new version is available. 
        // 2. You have articles which have not been published to Microsoft.
        $wpNotifications = array();
        $currentVersion = MSPH_PLUGIN_VERSION;
        $latestVersion = Util::get_latest_plugin_version();
        $profileFromDb = json_decode(Options::get_profile());
        if (Util::if_version_higher($latestVersion, $currentVersion)) {
            // Only show notification when there is new version and the new version's notification have not been dismissed.
            $dismissedNotificationVersion = Options::get_dismissed_notification_version();

            if (Util::if_version_higher($latestVersion, $dismissedNotificationVersion)) {
                array_push($wpNotifications, array(
                    'id' => 'wordpress_1',
                    'canDismiss' => true,
                    'severity' => "information",
                    'wpNotificationType' => 'version_update',
                    'propertyBag' => array(
                        'title' => /* translator: Notification of update plugin */ __('A new version of Microsoft Start is available.', 'microsoft-start'),
                        "action" => "[{\"Text\":\"" ./* translator: Update button in banner */ __("Update Now", 'microsoft-start') . "\",\"Href\":\"/wp-admin/plugins.php?s=microsoft-start\",\"IsExternalLink\":false}]"
                    )
                ));
            }
        }
        if (Options::get_status() === 'active' && Options::get_publishOption() === 'editor' && !Options::get_dismissed_publish_to_MSPH_notification() && ($profileFromDb->partnerMetadata->paymentStatus->stripeAccountStatus ?? null) === 1) {
            // Only show notification when there is new article have not been publish to MSPH and the publish artile notification have not been dismissed.
            $posts = get_posts([
                'post_type' => 'post',
                'post_status' => ['publish'],
                'meta_query' => [
                    [
                        'key' => 'msn_id',
                        'compare' => 'NOT EXISTS'
                    ]
                ]
            ]);
            if (count($posts)) {
                array_push($wpNotifications, array(
                    'id' => 'wordpress_2',
                    'canDismiss' => true,
                    'severity' => "information",
                    'wpNotificationType' => 'publish_article',
                    'propertyBag' => array(
                        'title' => /* translator: Notification of publish post */ __("You have articles that haven't been published to Microsoft Start yet.", 'microsoft-start'),
                        "action" => "[{\"Text\":\"" ./* translator: Publish button in banner */ __("Publish", "microsoft-start") . "\",\"Href\":\"/wp-admin/edit.php\",\"IsExternalLink\":false}]"
                    )
                ));
            }
        }
        return $wpNotifications;
    }

    static function get_notification($type)
    {
        $cid = Options::get_CID();
        $wpNotifications = static::get_wp_notification();
        $body = array('information' => array());
        if ($cid) {
            $profile = json_decode(Options::get_profile(), true);
            $partnerId = $profile['partnerId'];
            $response = static::BaseRequest(
                "notification/items?accountId="
                    . Options::get_CID() .
                    "&partnerId="
                    . $partnerId .
                    "&scenario=" . ($type === 'sidebar' ? "1" : "0")
            );
            if (!$response || $response['response']['code'] != 200) {
                return array(
                    'information' => $wpNotifications
                );
            }
            $body = json_decode($response['body'], true);
        }

        $information = array_merge($body['information'], $wpNotifications);
        $body['information'] = $information;
        $type === 'sidebar' && Options::set_red_dot(array_key_exists('critical', $body) && is_array($body['critical']) ? count($body['critical']): 0);
        return $body;
    }

    static function dismissMSPHNotification($body)
    {
        $response = static::BaseRequest(
            "notification/items",
            ['method' => 'POST', 'body' => $body]
        );
        return $response;
    }

    /**
     * @param string $url request path.
     * @param array $config request config.
     * [
     *  method => GET | POST | PUT,
     *  body => array,
     *  auth => boolean, 
     *  header => array,
     *  timeout => number of seconds until timeout
     * ].
     */
    private static function BaseRequest($url, $config = [])
    {
        $url_arr = explode('?', $url);
        $path = $url_arr[0];
        $query_string = isset($url_arr[1]) ? $url_arr[1] : '';

        $method = isset($config['method']) ? $config['method'] : 'GET';
        $body = isset($config['body']) ? $config['body'] : null;
        $auth = isset($config['auth']) ? $config['auth'] : true;
        $timeout = isset($config['timeout']) ? $config['timeout'] : 5;
        $header = array_merge(array(
            "accept" => "*/*",
            "accept-language" => "en-US,en;q=0.9",
            "content-type" => "application/json"
        ),  isset($config['header']) ? $config['header'] : []);

        $query = $query_string ? explode('&', $query_string) : array();
        array_push($query, 'wrapodata=false');
        if ($auth) {
            array_push($query, 'scn=3rdPartyAuth');
            $header['Authorization'] = 'Bearer ' . TokenService::get_token();
        }


        $query_string = join('&', $query);
        $request_url = MSPH_SERVICE_URL . $path . '?' . $query_string;
        if ($method === 'GET') {
            $response = wp_remote_get($request_url, array('headers' => $header, 'timeout' => $timeout));
        } else if ($method === 'POST') {
            $response = wp_remote_post($request_url, array('headers' => $header, 'body' => json_encode($body), 'timeout' => $timeout));
        } else if ($method === 'PUT') {
            $response = wp_remote_request($request_url, array('method' => 'PUT', 'headers' => $header, 'body' => json_encode($body), 'timeout' => $timeout));
        }

        // check for WPError cases
        if ($response && array_key_exists('errors', $response)) {
            return null;
        }
        if ($auth && $response && ($response['response']['code'] == 401 || ($response['response']['code'] == 400 && strpos($response['body'], 'Invalid access token.')))) {
            TokenService::delete_token();
        }
        return $response ? $response : null;
    }
}
